searchData={"items":[{"type":"module","title":"Lua","doc":"`Lua` is an ergonomic interface to [Luerl](https://github.com/rvirding/luerl), aiming to be the best way to use Luerl from Elixir.","ref":"Lua.html"},{"type":"module","title":"Features - Lua","doc":"* `~LUA` sigil for validating Lua code at compile-time\n* `deflua` macro for exposing Elixir functions to Lua\n* Improved error messages and sandboxing\n* Deep setting/getting variables and state\n* Excellent documentation and guides for working with Luerl\n\n> #### Lua the Elixir library vs Lua the language {: .info}\n> When referring to this library, `Lua` will be stylized as a link.\n> \n> References to Lua the language will be in plaintext and not linked.","ref":"Lua.html#module-features"},{"type":"module","title":"Executing Lua - Lua","doc":"`Lua` can be run using the `eval!/2` function\n\n``` elixir\n    iex> {[4], _} =\n    ...>   Lua.eval!(~LUA\"\"\"\n    ...>   return 2 + 2\n    ...>   \"\"\")\n\n```","ref":"Lua.html#module-executing-lua"},{"type":"module","title":"Exposing Elixir functions to Lua - Lua","doc":"`Lua` provides the `deflua` macro for exposing Elixir functions to Lua\n\n``` elixir\ndefmodule MyAPI do\n  use Lua.API\n      \n  deflua double(v), do: 2 * v\nend\n    \nlua = Lua.new() |> Lua.load_api(MyAPI)\n\n{[10], _} =\n  Lua.eval!(lua, ~LUA\"\"\"\n  return double(5)\n  \"\"\")\n\n```","ref":"Lua.html#module-exposing-elixir-functions-to-lua"},{"type":"module","title":"Calling Lua functions from Elixir - Lua","doc":"`Lua` can be used to expose complex functions written in Elixir. In some cases, you may want to call Lua functions from Elixir. This can\nbe achieved with the `Lua.call_function!/3` function\n\n``` elixir\ndefmodule MyAPI do\n  use Lua.API, scope: \"example\"\n\n  deflua foo(value), state do\n    Lua.call_function!(state, [:string, :lower], [value])\n  end\nend\n\nlua = Lua.new() |> Lua.load_api(MyAPI)\n\n{[\"wow\"], _} = Lua.eval!(lua, ~LUA\"return example.foo(\\\"WOW\\\")\")\n```","ref":"Lua.html#module-calling-lua-functions-from-elixir"},{"type":"module","title":"Modify Lua state from Elixir - Lua","doc":"You can also use `Lua` to modify the state of the lua environment inside your Elixir code. Imagine you have a queue module that you\nwant to implement in Elixir, with the queue stored in a global variable\n\n``` elixir\ndefmodule Queue do\n  use Lua.API, scope: \"q\"\n  \n  deflua push(v), state do\n    # Pull out the global variable \"my_queue\" from lua\n    queue = Lua.get!(state, [:my_queue])\n    \n    # Call the Lua function table.insert(table, value)\n    {[], state} = Lua.call_function!(state, [:table, :insert], [queue, v])\n    \n    # Return the modified lua state with no return values\n    {[], state}\n  end\nend\n\nlua = Lua.new() |> Lua.load_api(Queue)\n\n{[queue], _} =\n  Lua.eval!(lua, \"\"\"\n  my_queue = {}\n\n  q.push(\"first\")\n  q.push(\"second\")\n\n  return my_queue\n  \"\"\")\n  \n[\"first\", \"second\"] = Lua.Table.as_list(queue)\n```","ref":"Lua.html#module-modify-lua-state-from-elixir"},{"type":"module","title":"Credits - Lua","doc":"`Lua` piggy-backs off of Robert Virding's [Luerl](https://github.com/rvirding/luerl) project, which implements a Lua lexer, parser, and full-blown Lua virtual machine that runs inside the BEAM.","ref":"Lua.html#module-credits"},{"type":"function","title":"Lua.call_function!/3","doc":"Calls a function in Lua's state\n\n    iex> {[ret], _lua} = Lua.call_function!(Lua.new(), [:string, :lower], [\"HELLO ROBERT\"])\n    iex> ret\n    \"hello robert\"\n\nReferences to functions can also be passed\n\n    iex> {[ref], lua} = Lua.eval!(\"return string.lower\")\n    iex> {[ret], _lua} = Lua.call_function!(lua, ref, [\"FUNCTION REF\"])\n    iex> ret\n    \"function ref\"\n\nThis is also useful for executing Lua function's inside of Elixir APIs\n\n```elixir\ndefmodule MyAPI do\n  use Lua.API, scope: \"example\"\n\n  deflua foo(value), state do\n    Lua.call_function!(state, [:string, :lower], [value])\n  end\nend\n\nlua = Lua.new() |> Lua.load_api(MyAPI)\n\n{[\"wow\"], _} = Lua.eval!(lua, \"return example.foo(\"WOW\")\")\n```","ref":"Lua.html#call_function!/3"},{"type":"function","title":"Lua.decode!/2","doc":"Decodes a Lua value from its internal form\n\n    iex> {encoded, lua} = Lua.encode!(Lua.new(), %{a: 1})\n    iex> Lua.decode!(lua, encoded)\n    [{\"a\", 1}]","ref":"Lua.html#decode!/2"},{"type":"function","title":"Lua.encode!/2","doc":"Encodes a Lua value into its internal form\n\n    iex> {encoded, _} = Lua.encode!(Lua.new(), %{a: 1})\n    iex> encoded\n    {:tref, 14}","ref":"Lua.html#encode!/2"},{"type":"function","title":"Lua.eval!/2","doc":"Evaluates the Lua script, returning any returned values and the updated\nLua environment\n\n    iex> {[42], _} = Lua.eval!(Lua.new(), \"return 42\")\n\n\n`eval!/2` can also evaluate chunks by passing instead of a script. As a\npeformance optimization, it is recommended to call `load_chunk/2` if you\nwill be executing a chunk many times, but it is not necessary.\n\n    iex> {[4], _} = Lua.eval!(~LUA[return 2 + 2]c)","ref":"Lua.html#eval!/2"},{"type":"function","title":"Lua.get!/2","doc":"Gets a table value in Lua\n\n    iex> state = Lua.set!(Lua.new(), [:hello], \"world\")\n    iex> Lua.get!(state, [:hello])\n    \"world\"\n\nWhen a value doesn't exist, it returns nil\n\n    iex> Lua.get!(Lua.new(), [:nope])\n    nil\n\nIt can also get nested values\n\n    iex> state = Lua.set!(Lua.new(), [:a, :b, :c], \"nested\")\n    iex> Lua.get!(state, [:a, :b, :c])\n    \"nested\"","ref":"Lua.html#get!/2"},{"type":"function","title":"Lua.load_api/3","doc":"Inject functions written with the `deflua` macro into the Lua runtime.\n\nSee `Lua.API` for more information on writing api modules","ref":"Lua.html#load_api/3"},{"type":"function","title":"Lua.load_chunk/2","doc":"Similar to `:luerl_new.load/3`, it takes a `t:Lua.Chunk.t`, and loads\nit into state so that it can be evaluated via `eval!/2`\n\n    iex> {%Lua.Chunk{}, %Lua{}} = Lua.load_chunk(Lua.new(), ~LUA[return 2 + 2]c)","ref":"Lua.html#load_chunk/2"},{"type":"function","title":"Lua.load_file!/2","doc":"Loads a Lua file into the environment. Any values returned in the global\nscope are thrown away.\n\nMimics the functionality of Lua's [dofile](https://www.lua.org/manual/5.4/manual.html#pdf-dofile)","ref":"Lua.html#load_file!/2"},{"type":"function","title":"Lua.new/1","doc":"Initializes a Lua VM sandbox\n\n    iex> Lua.new()\n\nBy default, the following Lua functions are sandboxed.\n\n* `[:io]`\n* `[:file]`\n* `[:os, :execute]`\n* `[:os, :exit]`\n* `[:os, :getenv]`\n* `[:os, :remove]`\n* `[:os, :rename]`\n* `[:os, :tmpname]`\n* `[:package]`\n* `[:load]`\n* `[:loadfile]`\n* `[:require]`\n* `[:dofile]`\n* `[:load]`\n* `[:loadfile]`\n* `[:loadstring]`\n\nTo disable, use the `sandboxed` option, passing an empty list\n\n    iex> Lua.new(sandboxed: [])\n\nAlternatively, you can pass your own list of functions to sandbox. This is equivalent to calling\n`Lua.sandbox/2`.\n\n    iex> Lua.new(sandboxed: [[:os, :exit]])","ref":"Lua.html#new/1"},{"type":"function","title":"Options - Lua.new/1","doc":"* `:sandboxed` - list of paths to be sandboxed, e.g. `sandboxed: [[:require], [:os, :exit]]`\n* `:exclude` - list of paths to exclude from the sandbox, e.g. `exclude: [[:require], [:package]]`","ref":"Lua.html#new/1-options"},{"type":"function","title":"Lua.sandbox/2","doc":"Sandboxes the given path, swapping out the implementation with\na function that raises when called\n\n    iex> lua = Lua.new(sandboxed: [])\n    iex> Lua.sandbox(lua, [:os, :exit])","ref":"Lua.html#sandbox/2"},{"type":"function","title":"Lua.set!/3","doc":"Sets a table value in Lua. Nested keys will create\nintermediate tables\n\n    iex> Lua.set!(Lua.new(), [:hello], \"World\")\n\nIt can also set nested values\n\n    iex> Lua.set!(Lua.new(), [:a, :b, :c], [])\n\nThese table values are availble in lua scripts\n\n    iex> lua = Lua.set!(Lua.new(), [:a, :b, :c], \"nested!\")\n    iex> {result, _} = Lua.eval!(lua, \"return a.b.c\")\n    iex> result\n    [\"nested!\"]","ref":"Lua.html#set!/3"},{"type":"function","title":"Lua.set_lua_paths/2","doc":"Sets the path patterns that Luerl will look in when requiring Lua scripts. For example,\nif you store Lua files in your application's priv directory:\n\n    iex> lua = Lua.new(exclude: [[:package], [:require]])\n    iex> Lua.set_lua_paths(lua, [\"myapp/priv/lua/?.lua\", \"myapp/lua/?/init.lua\"])\n\nNow you can use the [Lua require](https://www.lua.org/pil/8.1.html) function to import\nthese scripts\n\n> #### Warning {: .warning}\n> In order to use `Lua.set_lua_paths/2`, the following functions cannot be sandboxed:\n> * `[:package]`\n> * `[:require]`\n>\n> By default these are sandboxed, see the `:exclude` option in `Lua.new/1` to allow them.","ref":"Lua.html#set_lua_paths/2"},{"type":"macro","title":"Lua.sigil_LUA/2","doc":"Write Lua code that is parsed at compile-time.\n\n    iex> ~LUA\"return 2 + 2\"\n    \"return 2 + 2\"\n\nIf the code cannot be lexed and parsed, it raises a `Lua.CompilerException`\n\n    #iex> ~LUA\":not_lua\"\n    ** (Lua.CompilerException) Failed to compile Lua!\n\nAs an optimization, the `c` modifier can be used to return a pre-compiled Lua chunk\n\n    iex> ~LUA\"return 2 + 2\"c","ref":"Lua.html#sigil_LUA/2"},{"type":"type","title":"Lua.t/0","doc":"","ref":"Lua.html#t:t/0"},{"type":"behaviour","title":"Lua.API","doc":"Defines the Behaviour for defining a Lua API\n\nTo create a module that exports functions to the global scope\n\n    defmodule MyAPI do\n      use Lua.API\n\n      # Can be called via `print(\"hi\")` in lua\n      deflua print(msg), do: IO.puts msg\n    end\n\nOptionally, you can provide a scope\n\n    defmodule SpecificAPI do\n      use Lua.API, scope: \"namespace.domain\"\n\n      # Can be called via `namespace.domain.foo(5)` in lua\n      deflua foo(v), do: v\n    end\n\nYou can access Lua state\n\n    defmodule State do\n      use Lua.API\n\n      deflua bar(name), state do\n        # Pull's the value of `number` out of state\n        val = Lua.get!(state, [:number])\n\n        2 * val\n      end\n    end\n\nRegular functions are not exported\n\n    defmodule SpecificAPI do\n      use Lua.API\n\n      # Won't be exposed\n      def baz(v), do: v\n    end","ref":"Lua.API.html"},{"type":"behaviour","title":"Installing an API - Lua.API","doc":"A `Lua.API` can provide an optional `install/1` callback, which\ncan run arbitrary Lua code or change the `Lua` state in any way.\n\nAn `install/1` callback takes a `t:Lua.t/0` and should either return a\nLua script to be evaluated, a `t:Lua.Chunk.t/0`, or return a new `t:Lua.t/0`\n\n    defmodule WithInstall do\n      use Lua.API, scope: \"install\"\n\n      @impl Lua.API\n      def install(lua) do\n        Lua.set!(lua, [:foo], \"bar\")\n      end\n    end\n\nIf you don't need to write Elixir, but want to execute some Lua\nto setup global variables, modify state, or expose some additonal\nAPIs, you can simply return a Lua chunk directly using the `c` modifier\non `Lua.sigil_LUA/2`\n\n    defmodule WithLua do\n      use Lua.API, scope: \"whoa\"\n\n      import Lua\n\n      @impl Lua.API\n      def install(_lua) do\n        ~LUA[print(\"Hello at install time!\")]c\n      end\n    end","ref":"Lua.API.html#module-installing-an-api"},{"type":"macro","title":"Lua.API.deflua/2","doc":"See `deflua/3`","ref":"Lua.API.html#deflua/2"},{"type":"macro","title":"Lua.API.deflua/3","doc":"Defines a function that can be exposed in Lua through `Lua.load_api/3`\n\n    deflua add_two(number) do\n      number + 2\n    end","ref":"Lua.API.html#deflua/3"},{"type":"macro","title":"Accessing state - Lua.API.deflua/3","doc":"Sometimes, you may want to access or modify the Lua environment in a `deflua`. This\ncan be done by using the following syntax\n\n    deflua get_value(key), state do\n      # Access the Lua environment\n      Lua.get!(lua, [key])\n    end\n\nTo modify and return new state, return a tuple\n\n    deflua set_value(key, value), state do\n      # Return nothing but modify the state\n      {[], Lua.set!(lua, [key])}\n    end","ref":"Lua.API.html#deflua/3-accessing-state"},{"type":"macro","title":"Variadic functions - Lua.API.deflua/3","doc":"Technically, all Lua functions are variadic, which means they can receive\na variable number of arguments. As a convenience, `Lua` applies your arguments to\n`deflua` functions so that they can be written in idiomatic Elixir.\n\nIf you need to handle variadic arguments, annotate the function with the `@variadic`\nmodule attribute.\n\n    @variadic true\n    deflua print(args) do\n      IO.puts(Enum.join(args, \" \"))\n    end\n\n> #### @variadic behavior {: .neutal}\n> When using the `@variadic` attribute, note that it is per-function. `Lua` will\n> reset this attribute after every function definition, so there is no need to\n> manually reset it yourself","ref":"Lua.API.html#deflua/3-variadic-functions"},{"type":"callback","title":"Lua.API.install/1","doc":"","ref":"Lua.API.html#c:install/1"},{"type":"macro","title":"Lua.API.runtime_exception!/1","doc":"Raises a runtime exception inside an API function, displaying contextual\ninformation about where the exception was raised.","ref":"Lua.API.html#runtime_exception!/1"},{"type":"callback","title":"Lua.API.scope/0","doc":"","ref":"Lua.API.html#c:scope/0"},{"type":"module","title":"Lua.Chunk","doc":"A pre-compiled [chunk](https://www.lua.org/pil/1.1.html) of Lua code\nthat can be executed at a future point in time","ref":"Lua.Chunk.html"},{"type":"type","title":"Lua.Chunk.t/0","doc":"","ref":"Lua.Chunk.html#t:t/0"},{"type":"module","title":"Lua.Table","doc":"In Lua, [tables](https://www.lua.org/pil/2.5.html) are the fundamental datastructure,\nwhich are used both as associative arrays (maps), and arrays (lists).\n\n`Lua.Table` provides some utilities for working with Lua tables when passed back to\nElixir.","ref":"Lua.Table.html"},{"type":"function","title":"Lua.Table.as_list/2","doc":"Converts a Lua table into a list. Assumes that the\ntable is correctly ordered.\n\n    iex> Lua.Table.as_list([{1, \"a\"}, {2, \"b\"}, {3, \"c\"}])\n    [\"a\", \"b\", \"c\"]\n\nTo ensure the list is ordered, you can pass the `:sort` option\n\n    iex> Lua.Table.as_list([{2, \"b\"}, {1, \"a\"}, {3, \"c\"}])\n    [\"b\", \"a\", \"c\"]\n\n    iex> Lua.Table.as_list([{2, \"b\"}, {1, \"a\"}, {3, \"c\"}], sort: true)\n    [\"a\", \"b\", \"c\"]","ref":"Lua.Table.html#as_list/2"},{"type":"function","title":"Lua.Table.as_map/1","doc":"Converts a Lua table into a map\n\n    iex> Lua.Table.as_map([{\"a\", 1}, {\"b\", 2}])\n    %{\"a\" => 1, \"b\" => 2}","ref":"Lua.Table.html#as_map/1"},{"type":"function","title":"Lua.Table.as_string/2","doc":"Converts a Lua table into a string representing\na Lua table literal\n\n    iex> Lua.Table.as_string([{\"a\", 1}, {\"b\", 2}])\n    \"{a = 1, b = 2}\"\n\n    iex> Lua.Table.as_string([{1, \"foo\"}, {2, \"bar\"}])\n    ~S[{\"foo\", \"bar\"}]","ref":"Lua.Table.html#as_string/2"},{"type":"function","title":"Options - Lua.Table.as_string/2","doc":"* `:userdata` - A 1-arity function used to format userdata. Defaults to `fn _ -> \" \"`","ref":"Lua.Table.html#as_string/2-options"},{"type":"function","title":"Lua.Table.deep_cast/1","doc":"Converts a Lua table into more \"native\" feeling lists and\nmaps, deeply traversing any sub-tables.\n\nIt uses the heuristic that maps with integer keys starting\nas 1 will be auto-cast into lists\n\n    iex> Lua.Table.deep_cast([{\"a\", 1}, {\"b\", [{1, 3}, {2, 4}]}])\n    %{\"a\" => 1, \"b\" => [3, 4]}","ref":"Lua.Table.html#deep_cast/1"},{"type":"exception","title":"Lua.CompilerException","doc":"","ref":"Lua.CompilerException.html"},{"type":"exception","title":"Lua.RuntimeException","doc":"","ref":"Lua.RuntimeException.html"},{"type":"extras","title":"Working with Lua","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Working with Lua\n\n```elixir\nMix.install([\n  {:lua, \"~> 0.0.16\"}\n])\n```","ref":"working-with-lua.html"},{"type":"extras","title":"Setup - Working with Lua","doc":"```elixir\nimport Lua, only: [sigil_LUA: 2]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nLua\n```","ref":"working-with-lua.html#setup"},{"type":"extras","title":"Executing Lua - Working with Lua","doc":"```elixir\ncode = ~LUA\"\"\"\n\n\"\"\"\n\n{value, %Lua{}} = Lua.eval!(code)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{[], #Lua<>}\n```","ref":"working-with-lua.html#executing-lua"},{"type":"extras","title":"Getting and Setting values - Working with Lua","doc":"```elixir\nlua =\n  Lua.new()\n  |> Lua.set!([:dave], \"Lucia\")\n\n{[\"Grohl\", \"Lucia\"], %Lua{} = lua} =\n  Lua.eval!(lua, ~LUA\"\"\"\n  local original = dave;\n\n  dave = \"Grohl\"\n\n  return dave, original\n  \"\"\")\n\nLua.get!(lua, [:dave])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\"Grohl\"\n```","ref":"working-with-lua.html#getting-and-setting-values"},{"type":"extras","title":"Exposing Elixir functions - Working with Lua","doc":"`Lua` provides a module-based abstraction for defining APIs that can be called from Lua functions. Using `use Lua.API` with `deflua` defines APIs that are then loaded into the environment with the `Lua.load_api/2` function.\n\nIn the example below, we load the `my_print` variadic function (a function that can take a variable number of arguments), and then call it from Lua.\n\n```elixir\ndefmodule Global do\n  use Lua.API\n\n  @variadic true\n  deflua my_print(args) do\n    IO.puts(Enum.join(args, \" \"))\n  end\nend\n\nLua.new()\n|> Lua.load_api(Global)\n|> Lua.eval!(~LUA\"\"\"\nmy_print(\"one\", \"two\", \"three\")\n\"\"\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\none two three\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{[], #Lua<>}\n```\n\nThere is quite a bit of flexibility for how functions can be defined. The module itself can define a scope under which all functions will be defined. In the previous example, no scope was defined, so functions were exposed in the globalscope.\n\nHere we'll define a scope, \"my.foo\", and expose functions \"bar\" and \"baz\" underneath it\n\n```elixir\ndefmodule MyFooAPI do\n  use Lua.API, scope: \"my.foo\"\n\n  deflua bar(string) do\n    \"bar \" <> string\n  end\n\n  deflua baz(string) do\n    \"baz \" <> string\n  end\nend\n\n{vals, %Lua{}} =\n  Lua.new()\n  |> Lua.load_api(MyFooAPI)\n  |> Lua.eval!(~LUA\"\"\"\n   return my.foo.bar(\"hi\"), my.foo.baz(\"hello\")\n  \"\"\")\n\nvals\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\"bar hi\", \"baz hello\"]\n```","ref":"working-with-lua.html#exposing-elixir-functions"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}